@@ -6,8 +6,10 @@ Python script for building documentation.
 To build the docs you must have all optional dependencies for pandas
 installed. See the installation instructions for a list of these.
 
-Note: currently latex builds do not work because of table formats that are not
-supported in the latex generation.
+<del>Note: currently latex builds do not work because of table formats that are not
+supported in the latex generation.</del>
+
+2014-01-30: Latex has some issues but 'latex_forced' works ok for 0.13.0-400 or so
 
 Usage
 -----
@@ -21,6 +23,8 @@ import os
 import shutil
 import sys
 import sphinx
+import argparse
+import jinja2
 
 os.environ['PYTHONPATH'] = '..'
 
@@ -77,7 +81,7 @@ def build_pandas():
         os.system('python setup.py clean')
         os.system('python setup.py build_ext --inplace')
         os.chdir('doc')
-        
+
 def build_prev(ver):
     if os.system('git checkout v%s' % ver) != 1:
         os.chdir('..')
@@ -103,7 +107,21 @@ def html():
     if os.system('sphinx-build -P -b html -d build/doctrees '
                  'source build/html'):
         raise SystemExit("Building HTML failed.")
+    try:
+        # remove stale file
+        os.system('cd build; rm -f html/pandas.zip;')
+    except:
+        pass
 
+def zip_html():
+    try:
+        print("\nZipping up HTML docs...")
+        # just in case the wonky build box doesn't have zip
+        # don't fail this.
+        os.system('cd build; rm -f html/pandas.zip; zip html/pandas.zip -r -q html/* ')
+        print("\n")
+    except:
+        pass
 
 def latex():
     check_build()
@@ -118,7 +136,33 @@ def latex():
 
         # Call the makefile produced by sphinx...
         if os.system('make'):
-            raise SystemExit("Rendering LaTeX failed.")
+            print("Rendering LaTeX failed.")
+            print("You may still be able to get a usable PDF file by going into 'build/latex'")
+            print("and executing 'pdflatex pandas.tex' for the requisite number of passes.")
+            print("Or using the 'latex_forced' target")
+            raise SystemExit
+
+        os.chdir('../..')
+    else:
+        print('latex build has not been tested on windows')
+
+def latex_forced():
+    check_build()
+    if sys.platform != 'win32':
+        # LaTeX format.
+        if os.system('sphinx-build -b latex -d build/doctrees '
+                     'source build/latex'):
+            raise SystemExit("Building LaTeX failed.")
+        # Produce pdf.
+
+        os.chdir('build/latex')
+
+        # Manually call pdflatex, 3 passes should ensure latex fixes up
+        # all the required cross-references and such.
+        os.system('pdflatex -interaction=nonstopmode pandas.tex')
+        os.system('pdflatex -interaction=nonstopmode pandas.tex')
+        os.system('pdflatex -interaction=nonstopmode pandas.tex')
+        raise SystemExit("You should check the file 'build/latex/pandas.pdf' for problems.")
 
         os.chdir('../..')
     else:
@@ -235,11 +279,13 @@ def _get_config():
 
 funcd = {
     'html': html,
+    'zip_html': zip_html,
     'upload_dev': upload_dev,
     'upload_stable': upload_stable,
     'upload_dev_pdf': upload_dev_pdf,
     'upload_stable_pdf': upload_stable_pdf,
     'latex': latex,
+    'latex_forced': latex_forced,
     'clean': clean,
     'auto_dev': auto_dev_build,
     'auto_debug': lambda: auto_dev_build(True),
@@ -252,22 +298,77 @@ small_docs = False
 # current_dir = os.getcwd()
 # os.chdir(os.path.dirname(os.path.join(current_dir, __file__)))
 
-if len(sys.argv) > 2:
-    ftype = sys.argv[1]
-    ver = sys.argv[2]
+import argparse
+argparser = argparse.ArgumentParser(description="""
+Pandas documentation builder
+""".strip())
 
-    if ftype == 'build_previous':
-        build_prev(ver)
-    if ftype == 'upload_previous':
-        upload_prev(ver)
-elif len(sys.argv) > 1:
-    for arg in sys.argv[1:]:
-        func = funcd.get(arg)
-        if func is None:
-            raise SystemExit('Do not know how to handle %s; valid args are %s' % (
-                arg, list(funcd.keys())))
-        func()
-else:
-    small_docs = False
-    all()
+# argparser.add_argument('-arg_name', '--arg_name',
+#                    metavar='label for arg help',
+#                    type=str|etc,
+#                    nargs='N|*|?|+|argparse.REMAINDER',
+#                    required=False,
+#                    #choices='abc',
+#                    help='help string',
+#                    action='store|store_true')
+
+# args = argparser.parse_args()
+
+#print args.accumulate(args.integers)
+
+def generate_index(api=True, single=False, **kwds):
+    from jinja2 import Template
+    with open("source/index.rst.template") as f:
+        t = Template(f.read())
+
+    with open("source/index.rst","w") as f:
+        f.write(t.render(api=api,single=single,**kwds))
+
+import argparse
+argparser = argparse.ArgumentParser(description="Pandas documentation builder",
+                                    epilog="Targets : %s" % funcd.keys())
+
+argparser.add_argument('--no-api',
+                   default=False,
+                   help='Ommit api and autosummary',
+                   action='store_true')
+argparser.add_argument('--single',
+                   metavar='FILENAME',
+                   type=str,
+                   default=False,
+                   help='filename of section to compile, e.g. "indexing"')
+
+def main():
+    args, unknown = argparser.parse_known_args()
+    sys.argv = [sys.argv[0]] + unknown
+    if args.single:
+        args.single = os.path.basename(args.single).split(".rst")[0]
+
+    if 'clean' in unknown:
+        args.single=False
+
+    generate_index(api=not args.no_api and not args.single, single=args.single)
+
+    if len(sys.argv) > 2:
+        ftype = sys.argv[1]
+        ver = sys.argv[2]
+
+        if ftype == 'build_previous':
+            build_prev(ver)
+        if ftype == 'upload_previous':
+            upload_prev(ver)
+    elif len(sys.argv) == 2:
+        for arg in sys.argv[1:]:
+            func = funcd.get(arg)
+            if func is None:
+                raise SystemExit('Do not know how to handle %s; valid args are %s' % (
+                    arg, list(funcd.keys())))
+            func()
+    else:
+        small_docs = False
+        all()
 # os.chdir(current_dir)
+
+if __name__ == '__main__':
+    import sys
+    sys.exit(main())
