@@ -10,7 +10,7 @@ import pandas.core.common as com
 import pandas.algos as algos
 import pandas.hashtable as htable
 import pandas.compat as compat
-
+from pandas.compat import filter, string_types
 
 def match(to_match, values, na_sentinel=-1):
     """
@@ -33,11 +33,20 @@ def match(to_match, values, na_sentinel=-1):
     match : ndarray of integers
     """
     values = com._asarray_tuplesafe(values)
-    if issubclass(values.dtype.type, compat.string_types):
+    if issubclass(values.dtype.type, string_types):
         values = np.array(values, dtype='O')
 
     f = lambda htype, caster: _match_generic(to_match, values, htype, caster)
-    return _hashtable_algo(f, values.dtype)
+    result = _hashtable_algo(f, values.dtype)
+
+    if na_sentinel != -1:
+
+        # replace but return a numpy array
+        # use a Series because it handles dtype conversions properly
+        from pandas.core.series import Series
+        result = Series(result.ravel()).replace(-1,na_sentinel).values.reshape(result.shape)
+
+    return result
 
 
 def unique(values):
@@ -120,6 +129,10 @@ def factorize(values, sort=False, order=None, na_sentinel=-1):
 
     Returns
     -------
+    labels : the indexer to the original array
+    uniques : the unique values
+
+    note: an array of Periods will ignore sort as it returns an always sorted PeriodIndex
     """
     from pandas.tseries.period import PeriodIndex
     vals = np.asarray(values)
@@ -135,7 +148,20 @@ def factorize(values, sort=False, order=None, na_sentinel=-1):
     uniques = uniques.to_array()
 
     if sort and len(uniques) > 0:
-        sorter = uniques.argsort()
+        try:
+            sorter = uniques.argsort()
+        except:
+            # unorderable in py3 if mixed str/int
+            t = hash_klass(len(uniques))
+            t.map_locations(com._ensure_object(uniques))
+
+            # order ints before strings
+            ordered = np.concatenate([
+                np.sort(np.array([ e for i, e in enumerate(uniques) if f(e) ],dtype=object)) for f in [ lambda x: not isinstance(x,string_types),
+                                                                                                        lambda x: isinstance(x,string_types) ]
+                ])
+            sorter = com._ensure_platform_int(t.lookup(com._ensure_object(ordered)))
+
         reverse_indexer = np.empty(len(sorter), dtype=np.int_)
         reverse_indexer.put(sorter, np.arange(len(sorter)))
 
@@ -146,7 +172,7 @@ def factorize(values, sort=False, order=None, na_sentinel=-1):
         uniques = uniques.take(sorter)
 
     if is_datetime:
-        uniques = uniques.view('M8[ns]')
+        uniques = uniques.astype('M8[ns]')
     if isinstance(values, PeriodIndex):
         uniques = PeriodIndex(ordinal=uniques, freq=values.freq)
 
@@ -271,6 +297,7 @@ def rank(values, axis=0, method='average', na_option='keep',
         f, values = _get_data_algo(values, _rank2d_functions)
         ranks = f(values, axis=axis, ties_method=method,
                   ascending=ascending, na_option=na_option)
+
     return ranks
 
 
@@ -356,12 +383,22 @@ def _interpolate(a, b, fraction):
 
 
 def _get_data_algo(values, func_map):
+    mask = None
     if com.is_float_dtype(values):
         f = func_map['float64']
         values = com._ensure_float64(values)
     elif com.is_datetime64_dtype(values):
-        f = func_map['int64']
-        values = values.view('i8')
+
+        # if we have NaT, punt to object dtype
+        mask = com.isnull(values)
+        if mask.ravel().any():
+            f = func_map['generic']
+            values = com._ensure_object(values)
+            values[mask] = np.nan
+        else:
+            f = func_map['int64']
+            values = values.view('i8')
+
     elif com.is_integer_dtype(values):
         f = func_map['int64']
         values = com._ensure_int64(values)
