@@ -31,4 +31,4 @@ from pandas.core.common import save, load  # deprecated, remove in 0.13
 import pandas.core.datetools as datetools
 
 from pandas.core.config import (get_option, set_option, reset_option,
-                                describe_option, options)
+                                describe_option, option_context, options)
