@@ -119,6 +119,11 @@ pc_expand_repr_doc = """
     wrap-around across multiple "pages" if it's width exceeds `display.width`.
 """
 
+pc_show_dimensions_doc = """
+: boolean
+    Whether to print out dimensions at the end of DataFrame repr.
+"""
+
 pc_line_width_doc = """
 : int
     Deprecated.
@@ -163,14 +168,11 @@ pc_max_seq_items = """
     If set to None, the number of items to be printed is unlimited.
 """
 
-
 pc_max_info_rows_doc = """
 : int or None
-    Deprecated.
-"""
-
-pc_max_info_rows_deprecation_warning = """\
-max_info_rows has been deprecated, as reprs no longer use the info view.
+    df.info() will usually show null-counts for each column.
+    For large frames this can be quite slow. max_info_rows and max_info_cols
+    limit this null check only to frames with smaller dimensions then specified.
 """
 
 pc_large_repr_doc = """
@@ -245,6 +247,7 @@ with cf.config_prefix('display'):
     cf.register_option('encoding', detect_console_encoding(), pc_encoding_doc,
                        validator=is_text)
     cf.register_option('expand_frame_repr', True, pc_expand_repr_doc)
+    cf.register_option('show_dimensions', True, pc_show_dimensions_doc)
     cf.register_option('chop_threshold', None, pc_chop_threshold_doc)
     cf.register_option('max_seq_items', 100, pc_max_seq_items)
     cf.register_option('mpl_style', None, pc_mpl_style_doc,
@@ -266,9 +269,6 @@ cf.deprecate_option('display.height',
                     msg=pc_height_deprecation_warning,
                     rkey='display.max_rows')
 
-cf.deprecate_option('display.max_info_rows',
-                    msg=pc_max_info_rows_deprecation_warning)
-
 tc_sim_interactive_doc = """
 : boolean
     Whether to simulate interactive mode for purposes of testing
