@@ -3,7 +3,6 @@
 from pandas.tseries.tools import *
 from pandas.tseries.offsets import *
 from pandas.tseries.frequencies import *
-from dateutil import parser
 
 day = DateOffset()
 bday = BDay()
@@ -36,3 +35,23 @@ thisQuarterEnd = QuarterEnd(0)
 isBusinessDay = BDay().onOffset
 isMonthEnd = MonthEnd().onOffset
 isBMonthEnd = BMonthEnd().onOffset
+
+
+def _resolve_offset(freq, kwds):
+    if 'timeRule' in kwds or 'offset' in kwds:
+        offset = kwds.get('offset', None)
+        offset = kwds.get('timeRule', offset)
+        if isinstance(offset, compat.string_types):
+            offset = getOffset(offset)
+        warn = True
+    else:
+        offset = freq
+        warn = False
+
+    if warn:
+        import warnings
+        warnings.warn("'timeRule' and 'offset' parameters are deprecated,"
+                      " please use 'freq' instead",
+                      FutureWarning)
+
+    return offset
