@@ -28,7 +28,7 @@ from pandas.core.common import (isnull, notnull, _is_bool_indexer,
 from pandas.core.index import (Index, MultiIndex, InvalidIndexError,
                                _ensure_index, _handle_legacy_indexes)
 from pandas.core.indexing import (
-    _SeriesIndexer, _check_bool_indexer, _check_slice_bounds,
+    _check_bool_indexer, _check_slice_bounds,
     _is_index_slice, _maybe_convert_indices)
 from pandas.core import generic
 from pandas.core.internals import SingleBlockManager
@@ -95,6 +95,7 @@ class Series(generic.NDFrame):
 
     """
     One-dimensional ndarray with axis labels (including time series).
+
     Labels need not be unique but must be any hashable type. The object
     supports both integer- and label-based indexing and provides a host of
     methods for performing operations involving the index. Statistical
@@ -117,7 +118,8 @@ class Series(generic.NDFrame):
         dict.
     dtype : numpy.dtype or None
         If None, dtype will be inferred
-    copy : boolean, default False, copy input data
+    copy : boolean, default False
+        Copy input data
     """
     _metadata = ['name']
 
@@ -301,10 +303,20 @@ class Series(generic.NDFrame):
     def dtype(self):
         return self._data.dtype
 
+    @property
+    def dtypes(self):
+        """ for compat """
+        return self._data.dtype
+
     @property
     def ftype(self):
         return self._data.ftype
 
+    @property
+    def ftypes(self):
+        """ for compat """
+        return self._data.ftype
+
     @property
     def shape(self):
         return self._data.shape
@@ -435,11 +447,6 @@ class Series(generic.NDFrame):
 
         return values
 
-    def _xs(self, key, axis=0, level=None, copy=True):
-        return self.__getitem__(key)
-
-    xs = _xs
-
     def _ixs(self, i, axis=0):
         """
         Return the i-th value or values in the Series by location
@@ -802,7 +809,8 @@ class Series(generic.NDFrame):
 
     def reset_index(self, level=None, drop=False, name=None, inplace=False):
         """
-        Analogous to the DataFrame.reset_index function, see docstring there.
+        Analogous to the :meth:`pandas.DataFrame.reset_index` function, see
+        docstring there.
 
         Parameters
         ----------
@@ -1128,7 +1136,7 @@ class Series(generic.NDFrame):
 
         Parameters
         ----------
-        normalize: boolean, default False
+        normalize : boolean, default False
             If True then the Series returned will contain the relative
             frequencies of the unique values.
         sort : boolean, default True
@@ -1156,7 +1164,7 @@ class Series(generic.NDFrame):
         Parameters
         ----------
         sort : bool, default True
-            if True, will lexicographically sort values, if False skips
+            If True, will lexicographically sort values, if False skips
             sorting. Result ordering when ``sort=False`` is not defined.
 
         Returns
@@ -1339,11 +1347,7 @@ class Series(generic.NDFrame):
         -------
         desc : Series
         """
-        try:
-            from collections import Counter
-        except ImportError:  # pragma: no cover
-            # For Python < 2.7, we include a local copy of this:
-            from pandas.util.counter import Counter
+        from pandas.compat import Counter
 
         if self.dtype == object:
             names = ['count', 'unique']
@@ -1397,9 +1401,9 @@ class Series(generic.NDFrame):
         ----------
         other : Series
         method : {'pearson', 'kendall', 'spearman'}
-            pearson : standard correlation coefficient
-            kendall : Kendall Tau correlation coefficient
-            spearman : Spearman rank correlation
+            * pearson : standard correlation coefficient
+            * kendall : Kendall Tau correlation coefficient
+            * spearman : Spearman rank correlation
         min_periods : int, optional
             Minimum number of observations needed to have a valid result
 
@@ -1662,22 +1666,18 @@ class Series(generic.NDFrame):
 
         See Also
         --------
-        pandas.Series.order
+        Series.order
         """
-        sortedSeries = self.order(na_last=True, kind=kind,
-                                  ascending=ascending)
 
-        true_base = self.values
-        while true_base.base is not None:
-            true_base = true_base.base
+        # GH 5856/5863
+        if self._is_cached:
+            raise ValueError("This Series is a view of some other array, to "
+                             "sort in-place you must create a copy")
 
-        if (true_base is not None and
-                (true_base.ndim != 1 or true_base.shape != self.shape)):
-            raise TypeError('This Series is a view of some other array, to '
-                            'sort in-place you must create a copy')
+        result = self.order(na_last=True, kind=kind,
+                            ascending=ascending)
 
-        self._data = sortedSeries._data.copy()
-        self.index = sortedSeries.index
+        self._update_inplace(result)
 
     def sort_index(self, ascending=True):
         """
@@ -1753,10 +1753,10 @@ class Series(generic.NDFrame):
         Parameters
         ----------
         method : {'average', 'min', 'max', 'first'}
-            average: average rank of group
-            min: lowest rank in group
-            max: highest rank in group
-            first: ranks assigned in order they appear in the array
+            * average: average rank of group
+            * min: lowest rank in group
+            * max: highest rank in group
+            * first: ranks assigned in order they appear in the array
         na_option : {'keep'}
             keep: leave NA values where they are
         ascending : boolean, default True
@@ -2098,21 +2098,8 @@ class Series(generic.NDFrame):
         ----------
         values : list-like
             The sequence of values to test. Passing in a single string will
-            raise a ``TypeError``:
-
-                .. code-block:: python
-
-                   from pandas import Series
-                   s = Series(list('abc'))
-                   s.isin('a')
-
-            Instead, turn a single string into a ``list`` of one element:
-
-                .. code-block:: python
-
-                   from pandas import Series
-                   s = Series(list('abc'))
-                   s.isin(['a'])
+            raise a ``TypeError``. Instead, turn a single string into a
+            ``list`` of one element.
 
         Returns
         -------
@@ -2126,6 +2113,26 @@ class Series(generic.NDFrame):
         See Also
         --------
         pandas.DataFrame.isin
+
+        Examples
+        --------
+
+        >>> s = pd.Series(list('abc'))
+        >>> s.isin(['a', 'c', 'e'])
+        0     True
+        1    False
+        2     True
+        dtype: bool
+
+        Passing a single string as ``s.isin('a')`` will raise an error. Use
+        a list of one element instead:
+
+        >>> s.isin(['a'])
+        0     True
+        1    False
+        2    False
+        dtype: bool
+
         """
         if not com.is_list_like(values):
             raise TypeError("only list-like objects are allowed to be passed"
@@ -2174,7 +2181,7 @@ class Series(generic.NDFrame):
 
     @classmethod
     def from_csv(cls, path, sep=',', parse_dates=True, header=None,
-                 index_col=0, encoding=None):
+                 index_col=0, encoding=None, infer_datetime_format=False):
         """
         Read delimited file into Series
 
@@ -2193,6 +2200,10 @@ class Series(generic.NDFrame):
         encoding : string, optional
             a string representing the encoding to use if the contents are
             non-ascii, for python versions prior to 3
+        infer_datetime_format: boolean, default False
+            If True and `parse_dates` is True for a column, try to infer the
+            datetime format based on the first datetime string. If the format
+            can be inferred, there often will be a large parsing speed-up.
 
         Returns
         -------
@@ -2201,7 +2212,8 @@ class Series(generic.NDFrame):
         from pandas.core.frame import DataFrame
         df = DataFrame.from_csv(path, header=header, index_col=index_col,
                                 sep=sep, parse_dates=parse_dates,
-                                encoding=encoding)
+                                encoding=encoding,
+                                infer_datetime_format=infer_datetime_format)
         result = df.icol(0)
         result.index.name = result.name = None
         return result
@@ -2251,7 +2263,7 @@ class Series(generic.NDFrame):
         Returns
         -------
         valid : Series
-        inplace : bool (default False)
+        inplace : boolean, default False
             Do operation in place.
         """
         axis = self._get_axis_number(axis or 0)
@@ -2464,10 +2476,6 @@ Series._setup_axes(['index'], info_axis=0, stat_axis=0,
 Series._add_numeric_operations()
 _INDEX_TYPES = ndarray, Index, list, tuple
 
-# reinstall the SeriesIndexer
-# defined in indexing.py; pylint: disable=E0203
-Series._create_indexer('ix', _SeriesIndexer)
-
 #------------------------------------------------------------------------------
 # Supplementary functions
 
@@ -2531,7 +2539,10 @@ def _sanitize_array(data, index, dtype=None, copy=False,
                 else:
                     subarr = _try_cast(data, True)
         else:
-            subarr = _try_cast(data, True)
+            # don't coerce Index types
+            # e.g. indexes can have different conversions (so don't fast path them)
+            # GH 6140
+            subarr = _try_cast(data, not isinstance(data, Index))
 
         if copy:
             subarr = data.copy()
