@@ -13,8 +13,9 @@ from distutils.version import LooseVersion
 import pandas as pd
 import numpy as np
 
-from pandas import DataFrame, concat
 from pandas.core.common import PandasError
+from pandas.core.frame import DataFrame
+from pandas.tools.merge import concat
 
 try:
     import bq
@@ -203,9 +204,6 @@ def _parse_data(client, job, index_col=None, col_order=None):
     pagination API. We are using the most flexible iteration method
     that we could find in the bq.py/bigquery_client.py API's, but
     these have undergone large amounts of change recently.
-
-    We have encountered bugs with this functionality, see:
-    http://stackoverflow.com/questions/19145587/bq-py-not-paging-results
     """
 
     # dtype Map -
@@ -287,7 +285,7 @@ def _parse_data(client, job, index_col=None, col_order=None):
             kwds['startIndex'] = start_row
         data = client.apiclient.jobs().getQueryResults(**kwds).execute()
         if not data['jobComplete']:
-            raise BigqueryError('Job was not completed, or was invalid')
+            raise bigquery_client.BigqueryError('Job was not completed, or was invalid')
 
         # How many rows are there across all pages?
         # Note: This is presently the only reason we don't just use
@@ -350,6 +348,8 @@ def to_gbq(dataframe, destination_table, schema=None, col_order=None,
            if_exists='fail', **kwargs):
     """Write a DataFrame to a Google BigQuery table.
 
+    THIS IS AN EXPERIMENTAL LIBRARY
+
     If the table exists, the DataFrame will be appended. If not, a new table
     will be created, in which case the schema will have to be specified. By
     default, rows will be written in the order they appear in the DataFrame,
@@ -471,6 +471,8 @@ def read_gbq(query, project_id=None, destination_table=None, index_col=None,
              col_order=None, **kwargs):
     """Load data from Google BigQuery.
 
+    THIS IS AN EXPERIMENTAL LIBRARY
+
     The main method a user calls to load data from Google BigQuery into a
     pandas DataFrame. This is a simple wrapper for Google's bq.py and
     bigquery_client.py, which we use to get the source data. Because of this,
