@@ -1,10 +1,9 @@
-from pandas.util.py3compat import StringIO, BytesIO
+from pandas.compat import StringIO, BytesIO
 from datetime import date, datetime
 import csv
 import os
 import sys
 import re
-import unittest
 
 import nose
 
@@ -19,12 +18,12 @@ from pandas.io.parsers import (read_csv, read_table, read_fwf,
 from pandas.util.testing import (assert_almost_equal, assert_frame_equal,
                                  assert_series_equal, network)
 import pandas.lib as lib
-from pandas.util import py3compat
+from pandas import compat
 from pandas.lib import Timestamp
 import pandas.io.date_converters as conv
+import pandas.util.testing as tm
 
-
-class TestConverters(unittest.TestCase):
+class TestConverters(tm.TestCase):
 
     def setUp(self):
         self.years = np.array([2007, 2008])
