@@ -5,9 +5,10 @@ from pandas.compat import u
 from pandas.util.testing import network
 from pandas.util.testing import assert_frame_equal
 from numpy.testing.decorators import slow
-from pandas.io.wb import search, download
+from pandas.io.wb import search, download, get_countries
 import pandas.util.testing as tm
 
+
 class TestWB(tm.TestCase):
 
     @slow
@@ -30,7 +31,6 @@ class TestWB(tm.TestCase):
         expected.index = result.index
         assert_frame_equal(result, expected)
 
-
     @slow
     @network
     def test_wdi_download(self):
@@ -43,6 +43,12 @@ class TestWB(tm.TestCase):
         expected.index = result.index
         assert_frame_equal(result, pandas.DataFrame(expected))
 
+    @slow
+    @network
+    def test_wdi_get_countries(self):
+        result = get_countries()
+        self.assertTrue('Zimbabwe' in list(result['name']))
+
 
 if __name__ == '__main__':
     nose.runmodule(argv=[__file__, '-vvs', '-x', '--pdb', '--pdb-failure'],
