@@ -917,16 +917,24 @@ char *JSON_EncodeObject(JSOBJ obj, JSONObjectEncoder *enc, char *_buffer, size_t
   enc->end = enc->start + _cbBuffer;
   enc->offset = enc->start;
 
-  locale = strdup(setlocale(LC_NUMERIC, NULL));
-  if (!locale)
+  locale = setlocale(LC_NUMERIC, NULL);
+  if (strcmp(locale, "C"))
   {
-    SetError(NULL, enc, "Could not reserve memory block");
-    return NULL;
+    locale = strdup(locale);
+    if (!locale)
+    {
+      SetError(NULL, enc, "Could not reserve memory block");
+      return NULL;
+    }
+    setlocale(LC_NUMERIC, "C");
+    encode (obj, enc, NULL, 0);
+    setlocale(LC_NUMERIC, locale);
+    free(locale);
+  }
+  else 
+  {
+    encode (obj, enc, NULL, 0);
   }
-  setlocale(LC_NUMERIC, "C");
-  encode (obj, enc, NULL, 0);
-  setlocale(LC_NUMERIC, locale);
-  free(locale);
 
   Buffer_Reserve(enc, 1);
   if (enc->errorMsg)
