@@ -162,6 +162,14 @@ class TestSeriesPlots(tm.TestCase):
         ax = Series([200, 500]).plot(log=True, kind='bar')
         assert_array_equal(ax.yaxis.get_ticklocs(), expected)
 
+    @slow
+    def test_bar_ignore_index(self):
+        df = Series([1, 2, 3, 4], index=['a', 'b', 'c', 'd'])
+        ax = df.plot(kind='bar', use_index=False)
+        expected = ['0', '1', '2', '3']
+        result = [x.get_text() for x in ax.get_xticklabels()]
+        self.assertEqual(result, expected)
+
     def test_rotation(self):
         df = DataFrame(randn(5, 5))
         ax = df.plot(rot=30)
@@ -299,6 +307,10 @@ class TestSeriesPlots(tm.TestCase):
         _check_plot_works(autocorrelation_plot, self.ts)
         _check_plot_works(autocorrelation_plot, self.ts.values)
 
+        ax = autocorrelation_plot(self.ts, label='Test')
+        t = ax.get_legend().get_texts()[0].get_text()
+        self.assertEqual(t, 'Test')
+
     @slow
     def test_lag_plot(self):
         from pandas.tools.plotting import lag_plot
@@ -751,6 +763,7 @@ class TestDataFramePlots(tm.TestCase):
         _check_plot_works(scat, diagonal='kde')
         _check_plot_works(scat, diagonal='density')
         _check_plot_works(scat, diagonal='hist')
+        _check_plot_works(scat, range_padding=.1)
 
         def scat2(x, y, by=None, ax=None, figsize=None):
             return plt.scatter_plot(df, x, y, by, ax, figsize=None)
