@@ -29,6 +29,11 @@ class TestStringMethods(tm.TestCase):
 
     _multiprocess_can_split_ = True
 
+    def test_api(self):
+
+        # GH 6106
+        self.assert_(Series.str is None)
+
     def test_iter(self):
         # GH3638
         strs = 'google', 'wikimedia', 'wikipedia', 'wikitravel'
@@ -167,11 +172,15 @@ class TestStringMethods(tm.TestCase):
         tm.assert_almost_equal(result, exp)
 
     def test_contains(self):
-        values = ['foo', NA, 'fooommm__foo', 'mmm_']
+        values = ['foo', NA, 'fooommm__foo', 'mmm_', 'foommm[_]+bar']
         pat = 'mmm[_]+'
 
         result = strings.str_contains(values, pat)
-        expected = [False, np.nan, True, True]
+        expected = [False, NA, True, True, False]
+        tm.assert_almost_equal(result, expected)
+
+        result = strings.str_contains(values, pat, regex=False)
+        expected = [False, NA, False, False, True]
         tm.assert_almost_equal(result, expected)
 
         values = ['foo', 'xyz', 'fooommm__foo', 'mmm_']
@@ -357,7 +366,6 @@ class TestStringMethods(tm.TestCase):
         result = values.str.replace("(?<=\w),(?=\w)", ", ", flags=re.UNICODE)
         tm.assert_series_equal(result, exp)
 
-
     def test_repeat(self):
         values = Series(['a', 'b', NA, 'c', NA, 'd'])
 
@@ -456,7 +464,7 @@ class TestStringMethods(tm.TestCase):
         # Contains tests like those in test_match and some others.
 
         values = Series(['fooBAD__barBAD', NA, 'foo'])
-        er = [NA, NA] # empty row
+        er = [NA, NA]  # empty row
 
         result = values.str.extract('.*(BAD[_]+).*(BAD)')
         exp = DataFrame([['BAD__', 'BAD'], er, er])
@@ -540,6 +548,19 @@ class TestStringMethods(tm.TestCase):
         exp = DataFrame([['A', '1'], ['B', '2'], ['C', NA]], columns=['letter', 'number'])
         tm.assert_frame_equal(result, exp)
 
+    def test_get_dummies(self):
+        s = Series(['a|b', 'a|c', np.nan])
+        result = s.str.get_dummies('|')
+        expected = DataFrame([[1, 1, 0], [1, 0, 1], [0, 0, 0]],
+                             columns=list('abc'))
+        tm.assert_frame_equal(result, expected)
+
+        s = Series(['a;b', 'a', 7])
+        result = s.str.get_dummies(';')
+        expected = DataFrame([[0, 1, 1], [0, 1, 0], [1, 0, 0]],
+                             columns=list('7ab'))
+        tm.assert_frame_equal(result, expected)
+
     def test_join(self):
         values = Series(['a_b_c', 'c_d_e', np.nan, 'f_g_h'])
         result = values.str.split('_').str.join('_')
