@@ -202,25 +202,34 @@ plot_params = _Options()
 
 def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
                    diagonal='hist', marker='.', density_kwds=None,
-                   hist_kwds=None, **kwds):
+                   hist_kwds=None, range_padding=0.05, **kwds):
     """
     Draw a matrix of scatter plots.
 
     Parameters
     ----------
     frame : DataFrame
-    alpha : amount of transparency applied
-    figsize : a tuple (width, height) in inches
-    ax : Matplotlib axis object
-    grid : setting this to True will show the grid
-    diagonal : pick between 'kde' and 'hist' for
+    alpha : float, optional
+        amount of transparency applied
+    figsize : (float,float), optional
+        a tuple (width, height) in inches
+    ax : Matplotlib axis object, optional
+    grid : bool, optional
+        setting this to True will show the grid
+    diagonal : {'hist', 'kde'}
+        pick between 'kde' and 'hist' for
         either Kernel Density Estimation or Histogram
         plot in the diagonal
-    marker : Matplotlib marker type, default '.'
+    marker : str, optional
+        Matplotlib marker type, default '.'    
     hist_kwds : other plotting keyword arguments
         To be passed to hist function
     density_kwds : other plotting keyword arguments
         To be passed to kernel density estimate plot
+    range_padding : float, optional
+        relative extension of axis range in x and y
+        with respect to (x_max - x_min) or (y_max - y_min),
+        default 0.05
     kwds : other plotting keyword arguments
         To be passed to scatter function
 
@@ -250,6 +259,13 @@ def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
     # workaround because `c='b'` is hardcoded in matplotlibs scatter method
     kwds.setdefault('c', plt.rcParams['patch.facecolor'])
 
+    boundaries_list = []
+    for a in df.columns:
+        values = df[a].values[mask[a].values]
+        rmin_, rmax_ = np.min(values), np.max(values)
+        rdelta_ext = (rmax_ - rmin_) * range_padding / 2.
+        boundaries_list.append((rmin_ - rdelta_ext, rmax_+ rdelta_ext))
+
     for i, a in zip(lrange(n), df.columns):
         for j, b in zip(lrange(n), df.columns):
             ax = axes[i, j]
@@ -260,18 +276,25 @@ def scatter_matrix(frame, alpha=0.5, figsize=None, ax=None, grid=False,
                 # Deal with the diagonal by drawing a histogram there.
                 if diagonal == 'hist':
                     ax.hist(values, **hist_kwds)
+
                 elif diagonal in ('kde', 'density'):
                     from scipy.stats import gaussian_kde
                     y = values
                     gkde = gaussian_kde(y)
                     ind = np.linspace(y.min(), y.max(), 1000)
                     ax.plot(ind, gkde.evaluate(ind), **density_kwds)
+
+                ax.set_xlim(boundaries_list[i])
+
             else:
                 common = (mask[a] & mask[b]).values
 
                 ax.scatter(df[b][common], df[a][common],
                            marker=marker, alpha=alpha, **kwds)
 
+                ax.set_xlim(boundaries_list[j])
+                ax.set_ylim(boundaries_list[i])
+
             ax.set_xlabel('')
             ax.set_ylabel('')
 
@@ -672,13 +695,15 @@ def lag_plot(series, lag=1, ax=None, **kwds):
     return ax
 
 
-def autocorrelation_plot(series, ax=None):
+def autocorrelation_plot(series, ax=None, **kwds):
     """Autocorrelation plot for time series.
 
     Parameters:
     -----------
     series: Time series
     ax: Matplotlib axis object, optional
+    kwds : keywords
+        Options to pass to matplotlib plotting method
 
     Returns:
     -----------
@@ -705,7 +730,9 @@ def autocorrelation_plot(series, ax=None):
     ax.axhline(y=-z99 / np.sqrt(n), linestyle='--', color='grey')
     ax.set_xlabel("Lag")
     ax.set_ylabel("Autocorrelation")
-    ax.plot(x, y)
+    ax.plot(x, y, **kwds)
+    if 'label' in kwds:
+        ax.legend()
     ax.grid()
     return ax
 
@@ -1545,8 +1572,11 @@ class BarPlot(MPLPlot):
 
     def _post_plot_logic(self):
         for ax in self.axes:
-            str_index = [com.pprint_thing(key) for key in self.data.index]
-
+            if self.use_index:
+                str_index = [com.pprint_thing(key) for key in self.data.index]
+            else:
+                str_index = [com.pprint_thing(key) for key in
+                             range(self.data.shape[0])]
             name = self._get_index_name()
             if self.kind == 'bar':
                 ax.set_xlim([self.ax_pos[0] - 0.25, self.ax_pos[-1] + 1])
@@ -1967,7 +1997,7 @@ def hist_frame(data, column=None, by=None, grid=True, xlabelsize=None,
                xrot=None, ylabelsize=None, yrot=None, ax=None, sharex=False,
                sharey=False, figsize=None, layout=None, **kwds):
     """
-    Draw Histogram the DataFrame's series using matplotlib / pylab.
+    Draw histogram of the DataFrame's series using matplotlib / pylab.
 
     Parameters
     ----------
@@ -1995,6 +2025,8 @@ def hist_frame(data, column=None, by=None, grid=True, xlabelsize=None,
     kwds : other plotting keyword arguments
         To be passed to hist function
     """
+    import matplotlib.pyplot as plt
+
     if column is not None:
         if not isinstance(column, (list, np.ndarray)):
             column = [column]
@@ -2017,7 +2049,6 @@ def hist_frame(data, column=None, by=None, grid=True, xlabelsize=None,
 
         return axes
 
-    import matplotlib.pyplot as plt
     n = len(data.columns)
 
     if layout is not None:
