@@ -1,23 +1,24 @@
 from datetime import datetime, time, timedelta, date
 import sys
 import os
-import unittest
 
 import nose
 
 import numpy as np
+from pandas.compat import u
+import pandas.util.testing as tm
 
 try:
     import pandas.tseries.converter as converter
 except ImportError:
-    raise nose.SkipTest
+    raise nose.SkipTest("no pandas.tseries.converter, skipping")
 
 
 def test_timtetonum_accepts_unicode():
-    assert(converter.time2num("00:01") == converter.time2num(u"00:01"))
+    assert(converter.time2num("00:01") == converter.time2num(u("00:01")))
 
 
-class TestDateTimeConverter(unittest.TestCase):
+class TestDateTimeConverter(tm.TestCase):
 
     def setUp(self):
         self.dtc = converter.DatetimeConverter()
@@ -25,7 +26,7 @@ class TestDateTimeConverter(unittest.TestCase):
 
     def test_convert_accepts_unicode(self):
         r1 = self.dtc.convert("12:22", None, None)
-        r2 = self.dtc.convert(u"12:22", None, None)
+        r2 = self.dtc.convert(u("12:22"), None, None)
         assert(r1 == r2), "DatetimeConverter.convert should accept unicode"
 
     def test_conversion(self):
