@@ -1,26 +1,26 @@
 from pandas import *
 
 import numpy as np
-from itertools import izip
+from pandas.compat import zip, range, lzip
 from pandas.util.testing import rands
 import pandas.lib as lib
 
 N = 100000
 
-key1 = [rands(10) for _ in xrange(N)]
-key2 = [rands(10) for _ in xrange(N)]
+key1 = [rands(10) for _ in range(N)]
+key2 = [rands(10) for _ in range(N)]
 
-zipped = izip(key1, key2)
+zipped = lzip(key1, key2)
 
 
 def _zip(*args):
     arr = np.empty(N, dtype=object)
-    arr[:] = zip(*args)
+    arr[:] = lzip(*args)
     return arr
 
 
 def _zip2(*args):
-    return lib.list_to_object_array(zip(*args))
+    return lib.list_to_object_array(lzip(*args))
 
 index = MultiIndex.from_arrays([key1, key2])
 to_join = DataFrame({'j1': np.random.randn(100000)}, index=index)
