@@ -29,3 +29,11 @@ data = np.array(['foo', 'bar', 'baz'], dtype=object)
 """
 
 ctor_index_array_string = Benchmark('Index(data)', setup=setup)
+
+# index constructors
+setup = common_setup + """
+s = Series([Timestamp('20110101'),Timestamp('20120101'),Timestamp('20130101')]*1000)
+"""
+index_from_series_ctor = Benchmark('Index(s)', setup=setup)
+
+dtindex_from_series_ctor = Benchmark('DatetimeIndex(s)', setup=setup)
