@@ -26,6 +26,21 @@ everything and calculate a ration for the timing information.
 
 """
 
+# IMPORTANT NOTE
+#
+# This script should run on pandas versions at least as far back as 0.9.1.
+# devs should be able to use the latest version of this script with
+# any dusty old commit and expect it to "just work".
+# One way in which this is useful is when collecting historical data,
+# where writing some logic around this script may prove easier
+# in some cases then running vbench directly (think perf bisection).
+#
+# *please*, when you modify this script for whatever reason,
+# make sure you do not break it's functionality when running under older
+# pandas versions.
+# Note that depreaction warnings are turned off in main(), so there's
+# no need to change the actual code to supress such warnings.
+
 import shutil
 import os
 import sys
@@ -37,14 +52,29 @@ import re
 import random
 import numpy as np
 
+import pandas as pd
 from pandas import DataFrame, Series
 
 from suite import REPO_PATH
-
+VB_DIR = os.path.dirname(os.path.abspath(__file__))
 DEFAULT_MIN_DURATION = 0.01
 HEAD_COL="head[ms]"
 BASE_COL="base[ms]"
 
+try:
+    import git # gitpython
+except Exception:
+    print("Error: Please install the `gitpython` package\n")
+    sys.exit(1)
+
+class RevParseAction(argparse.Action):
+    def __call__(self, parser, namespace, values, option_string=None):
+        import subprocess
+        cmd = 'git rev-parse --short -verify {0}^{{commit}}'.format(values)
+        rev_parse = subprocess.check_output(cmd, shell=True)
+        setattr(namespace, self.dest, rev_parse.strip())
+
+
 parser = argparse.ArgumentParser(description='Use vbench to measure and compare the performance of commits.')
 parser.add_argument('-H', '--head',
                     help='Execute vbenches using the currently checked out copy.',
@@ -53,9 +83,17 @@ parser.add_argument('-H', '--head',
                     default=False)
 parser.add_argument('-b', '--base-commit',
                     help='The commit serving as performance baseline ',
-                    type=str)
+                    type=str, action=RevParseAction)
 parser.add_argument('-t', '--target-commit',
                     help='The commit to compare against the baseline (default: HEAD).',
+                    type=str, action=RevParseAction)
+parser.add_argument('--base-pickle',
+                    help='name of pickle file with timings data generated by a former `-H -d FILE` run. '\
+                    'filename must be of the form <hash>-*.* or specify --base-commit seperately',
+                    type=str)
+parser.add_argument('--target-pickle',
+                    help='name of pickle file with timings data generated by a former `-H -d FILE` run '\
+                    'filename must be of the form <hash>-*.* or specify --target-commit seperately',
                     type=str)
 parser.add_argument('-m', '--min-duration',
                     help='Minimum duration (in ms) of baseline test for inclusion in report (default: %.3f).' % DEFAULT_MIN_DURATION,
@@ -119,6 +157,11 @@ parser.add_argument('-S', '--stats',
                     action='store_true',
                     help='when specified with -N, prints the output of describe() per vbench results. '  )
 
+parser.add_argument('--temp-dir',
+                    metavar="PATH",
+                    default=None,
+                    help='Specify temp work dir to use. ccache depends on builds being invoked from consistent directory.'  )
+
 parser.add_argument('-q', '--quiet',
                     default=False,
                     action='store_true',
@@ -154,7 +197,8 @@ def profile_comparative(benchmarks):
     from vbench.db import BenchmarkDB
     from vbench.git import GitRepo
     from suite import BUILD, DB_PATH, PREPARE, dependencies
-    TMP_DIR = tempfile.mkdtemp()
+
+    TMP_DIR = args.temp_dir or tempfile.mkdtemp()
 
     try:
 
@@ -178,7 +222,7 @@ def profile_comparative(benchmarks):
 
         # ARGH. reparse the repo, without discarding any commits,
         # then overwrite the previous parse results
-        # prprint ("Slaughtering kittens..." )
+        # prprint("Slaughtering kittens...")
         (repo.shas, repo.messages,
          repo.timestamps, repo.authors) = _parse_commit_log(None,REPO_PATH,
                                                                 args.base_commit)
@@ -204,30 +248,82 @@ def profile_comparative(benchmarks):
 
         head_res = get_results_df(db, h_head)
         baseline_res = get_results_df(db, h_baseline)
-        ratio = head_res['timing'] / baseline_res['timing']
-        totals = DataFrame({HEAD_COL:head_res['timing'],
-                                BASE_COL:baseline_res['timing'],
-                                'ratio':ratio,
-                                'name':baseline_res.name},
-                                columns=[HEAD_COL, BASE_COL, "ratio", "name"])
-        totals = totals.ix[totals[HEAD_COL] > args.min_duration]
-            # ignore below threshold
-        totals = totals.dropna(
-        ).sort("ratio").set_index('name')  # sort in ascending order
 
-        h_msg =   repo.messages.get(h_head, "")
-        b_msg =   repo.messages.get(h_baseline, "")
+        report_comparative(head_res,baseline_res)
 
-        print_report(totals,h_head=h_head,h_msg=h_msg,
-                     h_baseline=h_baseline,b_msg=b_msg)
-
-        if args.outdf:
-            prprint("The results DataFrame was written to '%s'\n" %  args.outdf)
-            totals.save(args.outdf)
     finally:
         #        print("Disposing of TMP_DIR: %s" % TMP_DIR)
         shutil.rmtree(TMP_DIR)
 
+def prep_pickle_for_total(df, agg_name='median'):
+    """
+    accepts a datafram resulting from invocation with -H -d o.pickle
+    If multiple data columns are present (-N was used), the
+    `agg_name` attr of the datafram will be used to reduce
+    them to a single value per vbench, df.median is used by defa
+    ult.
+
+    Returns a datadrame of the form expected by prep_totals
+    """
+    def prep(df):
+        agg = getattr(df,agg_name)
+        df = DataFrame(agg(1))
+        cols = list(df.columns)
+        cols[0]='timing'
+        df.columns=cols
+        df['name'] = list(df.index)
+        return df
+
+    return prep(df)
+
+def prep_totals(head_res, baseline_res):
+    """
+    Each argument should be a dataframe with  'timing' and 'name' columns
+    where name is the name of the vbench.
+
+    returns a 'totals' dataframe, suitable as input for print_report.
+    """
+    head_res, baseline_res = head_res.align(baseline_res)
+    ratio = head_res['timing'] / baseline_res['timing']
+    totals = DataFrame({HEAD_COL:head_res['timing'],
+                        BASE_COL:baseline_res['timing'],
+                        'ratio':ratio,
+                        'name':baseline_res.name},
+                        columns=[HEAD_COL, BASE_COL, "ratio", "name"])
+    totals = totals.ix[totals[HEAD_COL] > args.min_duration]
+    # ignore below threshold
+    totals = totals.dropna(
+    ).sort("ratio").set_index('name')  # sort in ascending order
+    return totals
+
+def report_comparative(head_res,baseline_res):
+    try:
+        r=git.Repo(VB_DIR)
+    except:
+        import pdb
+        pdb.set_trace()
+
+    totals = prep_totals(head_res,baseline_res)
+
+    h_head = args.target_commit
+    h_baseline = args.base_commit
+    h_msg = b_msg = "Unknown"
+    try:
+        h_msg =  r.commit(h_head).message.strip()
+    except git.exc.BadObject:
+        pass
+    try:
+        b_msg =  r.commit(h_baseline).message.strip()
+    except git.exc.BadObject:
+        pass
+
+
+    print_report(totals,h_head=h_head,h_msg=h_msg,
+             h_baseline=h_baseline,b_msg=b_msg)
+
+    if args.outdf:
+        prprint("The results DataFrame was written to '%s'\n" %  args.outdf)
+        totals.save(args.outdf)
 
 def profile_head_single(benchmark):
     import gc
@@ -344,6 +440,10 @@ def print_report(df,h_head=None,h_msg="",h_baseline=None,b_msg=""):
 
     stats_footer = "\n"
     if args.stats :
+        try:
+            pd.options.display.expand_frame_repr=False
+        except:
+            pass
         stats_footer += str(df.T.describe().T) + "\n\n"
 
     s+= stats_footer
@@ -364,11 +464,6 @@ def print_report(df,h_head=None,h_msg="",h_baseline=None,b_msg=""):
 
 def main():
     from suite import benchmarks
-    # GitRepo wants exactly 7 character hash?
-    if args.base_commit:
-        args.base_commit = args.base_commit[:7]
-    if args.target_commit:
-        args.target_commit = args.target_commit[:7]
 
     if not args.log_file:
         args.log_file = os.path.abspath(
@@ -386,38 +481,22 @@ def main():
     random.seed(args.seed)
     np.random.seed(args.seed)
 
-    affinity_set = False
+    if args.base_pickle and args.target_pickle:
+        baseline_res = prep_pickle_for_total(pd.load(args.base_pickle))
+        target_res = prep_pickle_for_total(pd.load(args.target_pickle))
 
-    # try psutil first since it is more commonly present and better
-    # maintained.  Some people experienced problems with affinity package
-    # (see https://code.google.com/p/psutil/issues/detail?id=238 for more references)
-    try:
-        import psutil
-        if hasattr(psutil.Process, 'set_cpu_affinity'):
-            psutil.Process(os.getpid()).set_cpu_affinity([args.affinity])
-            affinity_set = True
-    except ImportError:
-        pass
+        report_comparative(target_res, baseline_res)
+        sys.exit(0)
 
-    if not affinity_set:
-        try:
-            import affinity
-            affinity.set_process_affinity_mask(0, args.affinity)
-            assert affinity.get_process_affinity_mask(0) == args.affinity
-            affinity_set = True
+    if args.affinity is not None:
+        try: # use psutil rather then stale affinity module. Thanks @yarikoptic
+            import psutil
+            if hasattr(psutil.Process, 'set_cpu_affinity'):
+                psutil.Process(os.getpid()).set_cpu_affinity([args.affinity])
+                print("CPU affinity set to %d" % args.affinity)
         except ImportError:
-            pass
-
-    if not affinity_set:
-        import warnings
-        warnings.warn("\n\n"
-              "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n"
-              "The 'affinity' or 'psutil' >= 0.5.0 modules are not available, results may be unreliable\n"
-              "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n\n"
-            )
-        time.sleep(2)
-    else:
-        print("CPU affinity set to %d" % args.affinity)
+            print("-a/--affinity specified, but the 'psutil' module is not available, aborting.\n")
+            sys.exit(1)
 
     print("\n")
     prprint("LOG_FILE = %s" % args.log_file)
@@ -499,10 +578,39 @@ def _parse_wrapper(base_commit):
 
 if __name__ == '__main__':
     args = parser.parse_args()
-    if not args.head and (not args.base_commit and not args.target_commit):
+    if (not args.head
+        and not (args.base_commit and args.target_commit)
+        and not (args.base_pickle and args.target_pickle)):
         parser.print_help()
-    else:
-        import warnings
-        warnings.filterwarnings('ignore',category=FutureWarning)
-        warnings.filterwarnings('ignore',category=DeprecationWarning)
-        main()
+        sys.exit(1)
+    elif ((args.base_pickle or args.target_pickle) and not
+        (args.base_pickle and args.target_pickle)):
+        print("Must specify Both --base-pickle and --target-pickle.")
+        sys.exit(1)
+
+    if ((args.base_pickle or args.target_pickle) and not
+        (args.base_commit and args.target_commit)):
+        if not args.base_commit:
+            print("base_commit not specified, Assuming base_pickle is named <commit>-foo.*")
+            args.base_commit = args.base_pickle.split('-')[0]
+        if not args.target_commit:
+            print("target_commit not specified, Assuming target_pickle is named <commit>-foo.*")
+            args.target_commit = args.target_pickle.split('-')[0]
+
+    import warnings
+    warnings.filterwarnings('ignore',category=FutureWarning)
+    warnings.filterwarnings('ignore',category=DeprecationWarning)
+
+    if args.base_commit and args.target_commit:
+        print("Verifying specified commits exist in repo...")
+        r=git.Repo(VB_DIR)
+        for c in [ args.base_commit, args.target_commit ]:
+            try:
+                msg =  r.commit(c).message.strip()
+            except git.BadObject:
+                print("The commit '%s' was not found, aborting..." % c)
+                sys.exit(1)
+            else:
+                print("%s: %s" % (c,msg))
+
+    main()
